import { Modifier } from "draft-js";
import { ContentBlock } from "draft-js";
import { genKey } from "draft-js";
import { CharacterMetadata } from "draft-js";
import { BlockMapBuilder } from "draft-js";
import { List, Repeat } from 'immutable';

var addBlock = function addBlock(editorState, selection, type, data, entityType) {
  var text = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : ' ';
  var currentContentState = editorState.getCurrentContent();
  var currentSelectionState = selection; // in case text is selected it is removed and then the block is appended

  var afterRemovalContentState = Modifier.removeRange(currentContentState, currentSelectionState, 'backward'); // deciding on the position to split the text

  var targetSelection = afterRemovalContentState.getSelectionAfter();
  var blockKeyForTarget = targetSelection.get('focusKey');
  var block = currentContentState.getBlockForKey(blockKeyForTarget);
  var insertionTargetSelection;
  var insertionTargetBlock; // In case there are no characters or entity or the selection is at the start it
  // is safe to insert the block in the current block.
  // Otherwise a new block is created (the block is always its own block)

  var isEmptyBlock = block.getLength() === 0 && block.getEntityAt(0) === null;
  var selectedFromStart = currentSelectionState.getStartOffset() === 0;

  if (isEmptyBlock || selectedFromStart) {
    insertionTargetSelection = targetSelection;
    insertionTargetBlock = afterRemovalContentState;
  } else {
    // the only way to insert a new seems to be by splitting an existing in to two
    insertionTargetBlock = Modifier.splitBlock(afterRemovalContentState, targetSelection); // the position to insert our blocks

    insertionTargetSelection = insertionTargetBlock.getSelectionAfter();
  } // TODO not sure why we need it …


  var newContentStateAfterSplit = Modifier.setBlockType(insertionTargetBlock, insertionTargetSelection, type); // creating a new ContentBlock including the entity with data
  // Entity will be created with a specific type, if defined, else will fall back to the ContentBlock type

  var contentStateWithEntity = newContentStateAfterSplit.createEntity(entityType || type, 'IMMUTABLE', data);
  var entityKey = contentStateWithEntity.getLastCreatedEntityKey();
  var charData = CharacterMetadata.create({
    entity: entityKey
  });
  var fragmentArray = [new ContentBlock({
    key: genKey(),
    type: type,
    text: text,
    characterList: List(Repeat(charData, text.length || 1)) // eslint-disable-line new-cap

  }), // new contentblock so we can continue wrting right away after inserting the block
  new ContentBlock({
    key: genKey(),
    type: 'unstyled',
    text: '',
    characterList: List() // eslint-disable-line new-cap

  })]; // create fragment containing the two content blocks

  var fragment = BlockMapBuilder.createFromArray(fragmentArray); // replace the contentblock we reserved for our insert

  return Modifier.replaceWithFragment(newContentStateAfterSplit, insertionTargetSelection, fragment);
};

export default addBlock;